/**
 * Author: Andrei Heidelbacher
 * Task: Network
 * Time complexity: O(V * K^2 + E)
 * Space complexity: O(V + E + K^2)
 * Expected score: 100 points
 */

#include <cmath>
#include <cassert>

#include <fstream>
#include <iomanip>
#include <vector>
#include <algorithm>

using namespace std;

const char IN_FILE[] = "network.in";
const char OUT_FILE[] = "network.out";
const double EPS = 1e-7;

class Edge {
 public:
  int v;
  double p[2];

  Edge(const int _v, const double _p[2]):
   v(_v) {
       p[0] = _p[0];
       p[1] = _p[1];
   }

  Edge(const int _v, const double _p0, const double _p1):
   v(_v) {
       p[0] = _p0;
       p[1] = _p1;
   }
};

typedef vector<vector<Edge>> Graph;
typedef vector<Edge>::const_iterator EdgeIt;
typedef vector<double> Equation;
typedef vector<Equation> System;

vector<double> gauss(System a) {
  int n = int(a.size()), m = int(a[0].size()) - 1;
  for (int i = 0, j = 0; i < n && j < m; ++i, ++j) {
    int k = i;
    for (; abs(a[k][j]) < EPS; ++k);
    assert(k < n);
    swap(a[i], a[k]);
    for (k = 0; k < n; ++k) {
      if (i == k)
        continue;
      double ratio = a[k][j] / a[i][j];
      for (int l = 0; l <= m; ++l)
        a[k][l] -= a[i][l] * ratio;
      a[k][j] = 0;
    }
  }
  vector<double> sol = vector<double>(m, 0);
  for (int i = n - 1, j; i >= 0; --i) {
    for (j = 0; j <= m && abs(a[i][j]) < EPS; ++j);
    assert(j < m);
    sol[j] = a[i][m] / a[i][j];
    for (int k = 0; k < i; ++k) {
      a[k][m] -= a[k][j] * sol[j];
      a[k][j] = 0;
    }
  }
  return sol;
}

void dfp(
    const Graph &graph,
    const int x,
    vector<int> &sign,
    vector<int> &topological) {
  ++sign[x];
  for (EdgeIt e = graph[x].begin(); e != graph[x].end(); ++e)
    if (sign[e->v] == 0)
      dfp(graph, e->v, sign, topological);
  topological.push_back(x);
}

void dfm(
    const Graph &graph,
    const int x,
    vector<int> &sign,
    vector<int> &comp) {
  --sign[x];
  comp.push_back(x);
  for (EdgeIt e = graph[x].begin(); e != graph[x].end(); ++e)
    if (sign[e->v] == 1)
      dfm(graph, e->v, sign, comp);
}

vector<vector<int>> kosaraju(const Graph &graph) {
  int v = int(graph.size());
  Graph graphT = Graph(v);
  for (int x = 0; x < v; ++x)
    for (EdgeIt e = graph[x].begin(); e != graph[x].end(); ++e)
      graphT[e->v].push_back(Edge(x, e->p));
  vector<int> sign = vector<int>(v, 0), topological;
  vector<vector<int>> comps;
  for (int x = 0; x < v; ++x)
    if (sign[x] == 0)
      dfp(graph, x, sign, topological);
  reverse(topological.begin(), topological.end());
  for (int i = 0; i < v; ++i) {
    const int x = topological[i];
    if (sign[x] == 1) {
      vector<int> comp;
      dfm(graphT, x, sign, comp);
      comps.push_back(comp);
    }
  }
  return comps;
}

inline int var(const int v, const int bit) {
  return (v << 1) ^ bit;
}

vector<double> solveComponent(
    const Graph &graph,
    const vector<int> &comp,
    vector<int> &index,
    const vector<double> &coef) {
  const int m = int(comp.size());
  for (int i = 0; i < m; ++i)
    index[comp[i]] = i;
  System system = System(2 * m, Equation(2 * m + 1, 0));
  for (int i = 0; i < m; ++i) {
    const int x = comp[i];
    const int degree = int(graph[x].size());
    for (int bit = 0; bit <= 1; ++bit) {
      const int eq = var(i, bit);
      system[eq][var(i, bit)] = degree;
      for (EdgeIt e = graph[x].begin(); e != graph[x].end(); ++e) {
        const int y = e->v;
        const double p[2] = {e->p[0], e->p[1]};
        if (index[y] == -1) {
          system[eq][2 * m] += p[bit] * coef[var(y, bit)];
          system[eq][2 * m] -= (1 - p[bit]) * coef[var(y, bit ^ 1)];
        } else {
          system[eq][var(index[y], bit)] -= p[bit];
          system[eq][var(index[y], bit ^ 1)] += 1 - p[bit];
        }
        system[eq][2 * m] += 1 - p[bit];
      }
    }
  }
  for (int i = 0; i < m; ++i)
    index[comp[i]] = -1;
  return gauss(system);
}

pair<double, double> solve(
    const Graph &graph,
    const int source,
    const int sink) {
  int v = int(graph.size());
  vector<vector<int>> comps = kosaraju(graph);
  reverse(comps.begin(), comps.end());
  vector<int> index = vector<int>(v, -1);
  vector<double> sol = vector<double>(2 * v, 0);
  sol[var(sink, 0)] = sol[var(sink, 1)] = 1;
  for (int i = 1; i < int(comps.size()); ++i) {
    vector<double> compSol = solveComponent(graph, comps[i], index, sol);
    for (int j = 0; j < int(comps[i].size()); ++j)
      for (int bit = 0; bit <= 1; ++bit)
        sol[var(comps[i][j], bit)] = compSol[var(j, bit)];
  }
  return make_pair(sol[var(source, 0)], sol[var(source, 1)]);
}

Graph read() {
  ifstream in(IN_FILE);
  int v, e;
  in >> v >> e;
  Graph graph = Graph(v);
  for (int i = 0; i < e; ++i) {
    int from, to;
    double p0, p1;
    in >> from >> to >> p0 >> p1;
    graph[from].push_back(Edge(to, p0, p1));
  }
  in.close();
  return graph;
}

void write(const double p0, const double p1) {
  ofstream out(OUT_FILE);
  out << fixed << setprecision(7) << p0 << "\n" << p1 << "\n";
  out.close();
}

int main() {
  const Graph graph = read();
  pair<double, double> p = solve(graph, 0, int(graph.size()) - 1);
  write(p.first, p.second);
  return 0;
}
