/**
 * Author: Andrei Heidelbacher
 * Task: Network
 * Time complexity: O(V^3 + E)
 * Space complexity: O(V^2 + E)
 * Expected score: 40 points
 */

#include <cmath>
#include <cassert>

#include <fstream>
#include <iomanip>
#include <vector>
#include <algorithm>

using namespace std;

const char IN_FILE[] = "network.in";
const char OUT_FILE[] = "network.out";
const double EPS = 1e-7;

class Edge {
 public:
  int v;
  double p[2];

  Edge(const int _v, const double _p[2]):
   v(_v),
   p({_p[0], p[1]}) {}

  Edge(const int _v, const double _p0, const double _p1):
   v(_v),
   p({_p0, _p1}) {}
};

typedef vector<vector<Edge>> Graph;
typedef vector<Edge>::const_iterator EdgeIt;
typedef vector<double> Equation;
typedef vector<Equation> System;

vector<double> gauss(System a) {
  int n = int(a.size()), m = int(a[0].size()) - 1;
  for (int i = 0, j = 0; i < n && j < m; ++i, ++j) {
    int k = i;
    for (; abs(a[k][j]) < EPS; ++k);
    assert(k < n);
    swap(a[i], a[k]);
    for (k = 0; k < n; ++k) {
      if (i == k)
        continue;
      double ratio = a[k][j] / a[i][j];
      for (int l = 0; l <= m; ++l)
        a[k][l] -= a[i][l] * ratio;
      a[k][j] = 0;
    }
  }
  vector<double> sol = vector<double>(m, 0);
  for (int i = n - 1, j; i >= 0; --i) {
    for (j = 0; j <= m && abs(a[i][j]) < EPS; ++j);
    assert(j < m);
    sol[j] = a[i][m] / a[i][j];
    for (int k = 0; k < i; ++k) {
      a[k][m] -= a[k][j] * sol[j];
      a[k][j] = 0;
    }
  }
  return sol;
}

inline int var(const int v, const int bit) {
  return (v << 1) ^ bit;
}

pair<double, double> solve(
    const Graph &graph,
    const int source,
    const int sink) {
  int v = int(graph.size());
  System system = System(2 * v, Equation(2 * v + 1, 0));
  for (int bit = 0; bit <= 1; ++bit)
    system[var(sink, bit)][var(sink, bit)] = system[var(sink, bit)][2 * v] = 1;
  for (int x = 0; x < v; ++x) {
    if (x == sink)
      continue;
    const int degree = int(graph[x].size());
    for (int bit = 0; bit <= 1; ++bit) {
      const int eq = var(x, bit);
      system[eq][var(x, bit)] = degree;
      for (EdgeIt e = graph[x].begin(); e != graph[x].end(); ++e) {
        const int y = e->v;
        const double p[2] = {e->p[0], e->p[1]};
        system[eq][var(y, bit)] -= p[bit];
        system[eq][var(y, bit ^ 1)] += 1 - p[bit];
        system[eq][2 * v] += 1 - p[bit];
      }
    }
  }
  vector<double> sol = gauss(system);
  return make_pair(sol[var(source, 0)], sol[var(source, 1)]);
}

Graph read() {
  ifstream in(IN_FILE);
  int v, e;
  in >> v >> e;
  Graph graph = Graph(v);
  for (int i = 0; i < e; ++i) {
    int from, to;
    double p0, p1;
    in >> from >> to >> p0 >> p1;
    graph[from].push_back(Edge(to, p0, p1));
  }
  in.close();
  return graph;
}

void write(const double p0, const double p1) {
  ofstream out(OUT_FILE);
  out << fixed << setprecision(7) << p0 << "\n" << p1 << "\n";
  out.close();
}

int main() {
  const Graph graph = read();
  pair<double, double> p = solve(graph, 0, int(graph.size()) - 1);
  write(p.first, p.second);
  return 0;
}
