#include <iostream>
#include <fstream>
#include <vector>
#include <array>

using namespace std;

double eps = 1e-6;

int cmp(double a, double b) {
    if (a + eps < b)
        return -1;
    if (b + eps < a)
        return 1;
    return 0;
}

class Graph {
  public:
    Graph(int size):
        m_edges(size),
        m_comp(size),
        m_dp(size) {
    }

    int size() const {
        return m_edges.size();
    }

    void addEdge(int from, int to, double x, double y) {
        m_edges[from].emplace_back(to, x, y);
    }

    pair<double, double> answer() {
        doConnectedComponents();
        for (auto &comp : m_components) {
            if (comp.size() == 1 && comp[0] == size() - 1) {
                m_dp[size() - 1][0] = m_dp[size() - 1][1] = 1.0;
                continue;
            }
            sort(comp.begin(), comp.end());

            for (int i = 0; i < int(comp.size()); ++i)
                m_index[comp[i]] = i;
            int K = comp.size();
            vector< vector<double> > gauss(2 * K, vector<double>(2 * K + 1, 0.0));
            for (auto &x : comp) {
                int with0 = m_index[x], with1 = with0 + K;
                gauss[with0][with0] = -1;
                gauss[with1][with1] = -1;
                double contrib = 1.0 / double(m_edges[x].size());
                for (auto &e : m_edges[x]) {
                    int to; double p1, p2;
                    tie(to, p1, p2) = e;

                    if (m_comp[to] == m_comp[x]) { // same component
                        // for 0
                        gauss[with0][m_index[to]] += contrib * p1;
                        gauss[with0][m_index[to] + K] += -contrib * (1 - p1);
                        gauss[with0][2 * K] += contrib * (1 - p1);

                        // for 1
                        gauss[with1][m_index[to] + K] += contrib * p2;
                        gauss[with1][m_index[to]] += -contrib * (1 - p2);
                        gauss[with1][2 * K] += contrib * (1 - p2);
                    } else { //we already know the dp
                        gauss[with0][2 * K] += contrib * (p1 * m_dp[to][0] + (1 - p1) * (1 - m_dp[to][1]));
                        gauss[with1][2 * K] += contrib * (p2 * m_dp[to][1] + (1 - p2) * (1 - m_dp[to][0]));
                    }
                }
            }
            auto results = doGauss(gauss);
            for (auto &x : comp) {
                m_dp[x][0] = results[m_index[x]];
                m_dp[x][1] = results[m_index[x] + K];
            }
        }
        return make_pair(m_dp[0][0], m_dp[0][1]);
    }

  private:
    void doConnectedComponents() {
        m_lowlink = vector<int>(size(), -1);
        m_index = vector<int>(size(), -1);
        m_time = 0;

        for (int i = 0; i < size(); ++i)
            if (m_lowlink[i] == -1)
                tarjan(i);
    }

    void tarjan(int node) {
        m_stack.push_back(node);
        m_lowlink[node] = m_index[node] = m_time++;

        for (auto &e : m_edges[node]) {
            auto to = get<0>(e);

            if (m_lowlink[to] == -1) {
                tarjan(to);
                m_lowlink[node] = min(m_lowlink[node], m_lowlink[to]);
            } else if (m_index[to] != -1) {
                m_lowlink[node] = min(m_lowlink[node], m_index[to]);
            }
        }

        if (m_lowlink[node] == m_index[node]) {
            vector<int> aux;
            int n;
            do {
                n = m_stack.back();
                m_stack.pop_back();
                m_index[n] = -1;
                m_comp[n] = m_components.size();
                aux.push_back(n);
            } while (node != n);
            m_components.push_back(aux);
        }
    }

    vector<double> doGauss(vector< vector<double> > &A) {
        int N = A.size();
        for (int i = 0; i < N; ++i) {
            for (int j = i; j < N; ++j)
                if (cmp(A[j][i], 0)) {
                    swap(A[j], A[i]);
                    break;
                }
            for (int j = 0; j < N; ++j)
                if (j != i)
                    for (int k = N; k >= i; --k)
                        A[j][k] -= A[j][i] / A[i][i] * A[i][k];
        }
        vector<double> result(N);
        for (int i = 0; i < N; ++i)
            result[i] = -A[i][N] / A[i][i];
        return result;
    }

    vector<vector<tuple<int, double, double>>> m_edges;
    vector<int> m_comp;
    vector< array<double, 2> > m_dp;

    vector< vector<int> > m_components;
    vector<int> m_lowlink, m_index, m_stack;
    int m_time;
};

int main() {
    ifstream cin("network.in");
    ofstream cout("network.out");

    int N, M; cin >> N >> M;

    Graph G(N);
    for (int i = 0; i < M; ++i) {
        int u, v; double p1, p2;
        cin >> u >> v >> p1 >> p2;
        G.addEdge(u, v, p1, p2);
    }

    auto a = G.answer();
    cout.setf(ios::fixed, ios::floatfield);
    cout << a.first << "\n" << a.second << "\n";
}
