/**
 * Author: Andrei Heidelbacher
 * Task: LuckyShuffle
 * Time complexity: O(VlogMOD + ElogMOD)
 * Space complexity: O(V + E)
 */

#include <fstream>
#include <vector>
#include <algorithm>
#include <queue>

using namespace std;

const char IN_FILE[] = "shuffle.in";
const char OUT_FILE[] = "shuffle.out";
const int MOD = 1000000007;

typedef vector<vector<int>> Graph;
typedef vector<int>::const_iterator EdgeIt;

Graph G, GT;
int V, E;
vector<int> Fact, InvFact;
int Ways;

inline int pow(int value, int power) {
  int result = 1;
  for (; power > 0; power >>= 1) {
    if (power & 1)
      result = (1LL * result * value) % MOD;
    value = (1LL * value * value) % MOD;
  }
  return result;
}

inline int inverse(const int value) {
  return pow(value, MOD - 2);
}

inline int choose(const int n, const int k) {
  if (n < 0 || k < 0 || k > n)
    return 0;
  const int denominator = (1LL * InvFact[k] * InvFact[n - k]) % MOD;
    return (1LL * Fact[n] * denominator) % MOD;
}

vector<int> bfs(const Graph &graph, const int source) {
  const int size = int(graph.size());
  queue<int> q;
  vector<int> distances = vector<int>(size, -1);
  distances[source] = 0;
  for (q.push(source); !q.empty(); q.pop()) {
    const int x = q.front();
    for (EdgeIt y = graph[x].begin(); y != graph[x].end(); ++y) {
      if (distances[*y] == -1) {
        distances[*y] = distances[x] + 1;
        q.push(*y);
      }
    }
  }
  return distances;
}

void preprocess() {
  Fact = InvFact = vector<int>(E + 1);
  Fact[0] = InvFact[0] = 1;
  for (int i = 1; i <= E; ++i) {
    Fact[i] = (1LL * i * Fact[i - 1]) % MOD;
    InvFact[i] = inverse(Fact[i]);
  }
}

void solve() {
  preprocess();
  vector<int> distances = bfs(GT, V - 1);
  int maxDist = 0;
  for (int x = 0; x < V; ++x)
    maxDist = max(maxDist, distances[x]);
  vector<vector<int>> layers = vector<vector<int>>(maxDist + 1);
  vector<int> layerSizes = vector<int>(maxDist + 1, 0);
  for (int x = 0; x < V; ++x) {
    layers[distances[x]].push_back(x);
    layerSizes[distances[x]] += int(G[x].size());
  }
  vector<int> dp = vector<int>(V, 0);
  dp[V - 1] = 1;
  for (int dist = 1, totalEdges = 0; dist <= maxDist; ++dist) {
    const int layerEdges = layerSizes[dist];
    totalEdges += layerEdges;
    const int layerCoef = choose(totalEdges, layerEdges);
    for (int i = 0; i < int(layers[dist].size()); ++i) {
      const int x = layers[dist][i];
      const int degree = int(G[x].size());
      const int nodeCoef = (1LL *
          ((1LL * layerCoef * Fact[degree - 1]) % MOD) *
          ((1LL * Fact[layerEdges - degree] * choose(layerEdges, degree)) % MOD)
      ) % MOD;
      for (EdgeIt y = G[x].begin(); y != G[x].end(); ++y)
        if (distances[*y] + 1 == distances[x])
          dp[x] = (dp[x] + 1LL * dp[*y] * nodeCoef) % MOD;
    }
  }
  Ways = dp[0];
}

void read() {
  ifstream in(IN_FILE);
  in >> V >> E;
  G = GT = Graph(V);
  for (int i = 0; i < E; ++i) {
    int from, to;
    in >> from >> to;
    --from;
    --to;
    G[from].push_back(to);
    GT[to].push_back(from);
  }
  in.close();
}

void write() {
  ofstream out(OUT_FILE);
  out << Ways << "\n";
  out.close();
}

int main() {
  read();
  solve();
  write();
  return 0;
}
