/**
 * Author: Andrei Heidelbacher
 * Task: LuckyShuffle
 * Time complexity: O(V * E + ElogMOD)
 * Space complexity: O(V + E)
 */

#include <fstream>
#include <vector>
#include <algorithm>
#include <queue>

using namespace std;

const char IN_FILE[] = "shuffle.in";
const char OUT_FILE[] = "shuffle.out";
const int MOD = 1000000007;

typedef vector<vector<int>> Graph;
typedef vector<int>::const_iterator EdgeIt;

Graph G, GT;
int V, E;
vector<int> Fact, InvFact;
int Ways;

inline int pow(int value, int power) {
  int result = 1;
  for (; power > 0; power >>= 1) {
    if (power & 1)
      result = (1LL * result * value) % MOD;
    value = (1LL * value * value) % MOD;
  }
  return result;
}

inline int inverse(const int value) {
  return pow(value, MOD - 2);
}

inline int choose(const int n, const int k) {
  if (n < 0 || k < 0 || k > n)
    return 0;
  const int denominator = (1LL * InvFact[k] * InvFact[n - k]) % MOD;
    return (1LL * Fact[n] * denominator) % MOD;
}

void countDfs(const int x, vector<bool> &visited, int &edgeCount) {
  visited[x] = true;
  edgeCount += int(G[x].size());
  for (EdgeIt y = G[x].begin(); y != G[x].end(); ++y)
    if (!visited[*y])
      countDfs(*y, visited, edgeCount);
}

void solveDfs(
    const int x,
    const vector<int> &edgeCount,
    const vector<int> &distances,
    vector<int> &dp) {
  if (x == V - 1) {
    dp[x] = 1;
    return;
  }
  const int degree = int(G[x].size());
  dp[x] = 0;
  for (EdgeIt y = G[x].begin(); y != G[x].end(); ++y) {
    if (dp[*y] == -1)
      solveDfs(*y, edgeCount, distances, dp);
    if (distances[*y] + 1 == distances[x]) {
      const int combineEdges = (1LL *
          choose(edgeCount[x], degree) *
          choose(edgeCount[x] - degree, edgeCount[*y])
      ) % MOD;

      dp[x] = (dp[x] + 1LL *
          ((1LL * dp[*y] * Fact[edgeCount[x] - edgeCount[*y] - degree]) % MOD) *
          ((1LL * combineEdges * Fact[degree - 1]) % MOD)
      ) % MOD;
    }
  }
}

void preprocess() {
  Fact = InvFact = vector<int>(E + 1);
  Fact[0] = InvFact[0] = 1;
  for (int i = 1; i <= E; ++i) {
    Fact[i] = (1LL * i * Fact[i - 1]) % MOD;
    InvFact[i] = inverse(Fact[i]);
  }
}

vector<int> bfs(const int source) {
  queue<int> q;
  vector<int> distances = vector<int>(V, -1);
  distances[source] = 0;
  for (q.push(source); !q.empty(); q.pop()) {
    const int x = q.front();
    for (EdgeIt y = GT[x].begin(); y != GT[x].end(); ++y) {
      if (distances[*y] == -1) {
        distances[*y] = distances[x] + 1;
        q.push(*y);
      }
    }
  }
  return distances;
}

void solve() {
  preprocess();
  vector<int> edgeCount = vector<int>(V, 0);
  for (int x = 0; x < V; ++x) {
    vector<bool> visited = vector<bool>(V, false);
    countDfs(x, visited, edgeCount[x]);
  }
  vector<int> distances = bfs(V - 1);
  vector<int> dp = vector<int>(V, -1);
  solveDfs(0, edgeCount, distances, dp);
  Ways = dp[0];
}

void read() {
  ifstream in(IN_FILE);
  in >> V >> E;
  G = GT = Graph(V);
  for (int i = 0; i < E; ++i) {
    int from, to;
    in >> from >> to;
    --from;
    --to;
    G[from].push_back(to);
    GT[to].push_back(from);
  }
  in.close();
}

void write() {
  ofstream out(OUT_FILE);
  out << Ways << "\n";
  out.close();
}

int main() {
  read();
  solve();
  write();
  return 0;
}
